/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef IMPL_TYPE_CAMERAOBJECTS_H
#define IMPL_TYPE_CAMERAOBJECTS_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint8.h"
#include "impl_type_float.h"

struct CameraObjects {
    ::UInt8 sensor_id{ 0U };
    ::UInt8 label{ 0U };
    ::Float type_confidence{ 0.0F };
    ::Float existence_confidence{ 0.0F };
    ::Float general_object_probilibty{ 0.0F };
    ::UInt8 sub_type{ 0U };
    ::UInt8 track_id{ 0U };
    ::UInt8 track_age_time_second{ 0U };
    ::UInt8 last_measure_update_timestamp_ns{ 0U };
    ::UInt8 current_prediction_timestamp_ns{ 0U };
    ::UInt8 motion_status{ 0U };
    ::Float motion_unknown_score{ 0.0F };
    ::Float motion_static_score{ 0.0F };
    ::Float motion_stopped_score{ 0.0F };
    ::Float motion_moving_score{ 0.0F };
    ::Float motion_crossing_score{ 0.0F };
    ::UInt8 maintenance_status{ 0U };
    ::UInt8 occlusion_status{ 0U };
    ::UInt8 sensor_source{ 0U };
    ::UInt8 derived_type{ 0U };
    ::UInt8 is_right_edge_detected{ 0U };
    ::UInt8 is_with_ped{ 0U };
    ::UInt8 umbrella{ 0U };
    ::UInt8 call{ 0U };
    ::UInt8 phone{ 0U };
    ::UInt8 wave{ 0U };
    ::UInt8 standing{ 0U };
    ::UInt8 is_on_bike{ 0U };
    ::UInt8 point_cloud_timestamp_ns{ 0U };
    ::Float track_range{ 0.0F };
    ::Float track_range_rate{ 0.0F };
    ::Float track_range_accel_rate{ 0.0F };
    ::Float track_lat_rate{ 0.0F };
    ::UInt8 color_label{ 0U };
    ::UInt8 sub_type_probs{ 0U };
    ::UInt8 feature{ 0U };
    ::UInt8 matched_sensor_object_info{ 0U };
    ::UInt8 size_info{ 0U };
    ::UInt8 polygon_box{ 0U };
    ::UInt8 direction_info{ 0U };
    ::UInt8 bv_corners{ 0U };
    ::UInt8 motion_info{ 0U };
    ::UInt8 camera_bbox_info{ 0U };
    ::UInt8 general_merge_info{ 0U };
    ::UInt8 bbox_motion_info{ 0U };
    ::UInt8 attribute_{ 0U };
    ::UInt8 automobile_light_status{ 0U };
    ::UInt8 right_edge_line{ 0U };
    ::UInt8 non_automobile_vehicle_light_status{ 0U };
    ::UInt8 lower_body_direction{ 0U };
    ::UInt8 upper_body_direction{ 0U };
    ::UInt8 face_direction{ 0U };
    ::UInt8 intention{ 0U };
    ::UInt8 depth_point{ 0U };
    ::UInt8 od_rd_info{ 0U };
    ::UInt8 debug_info{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(sensor_id);
        fun(label);
        fun(type_confidence);
        fun(existence_confidence);
        fun(general_object_probilibty);
        fun(sub_type);
        fun(track_id);
        fun(track_age_time_second);
        fun(last_measure_update_timestamp_ns);
        fun(current_prediction_timestamp_ns);
        fun(motion_status);
        fun(motion_unknown_score);
        fun(motion_static_score);
        fun(motion_stopped_score);
        fun(motion_moving_score);
        fun(motion_crossing_score);
        fun(maintenance_status);
        fun(occlusion_status);
        fun(sensor_source);
        fun(derived_type);
        fun(is_right_edge_detected);
        fun(is_with_ped);
        fun(umbrella);
        fun(call);
        fun(phone);
        fun(wave);
        fun(standing);
        fun(is_on_bike);
        fun(point_cloud_timestamp_ns);
        fun(track_range);
        fun(track_range_rate);
        fun(track_range_accel_rate);
        fun(track_lat_rate);
        fun(color_label);
        fun(sub_type_probs);
        fun(feature);
        fun(matched_sensor_object_info);
        fun(size_info);
        fun(polygon_box);
        fun(direction_info);
        fun(bv_corners);
        fun(motion_info);
        fun(camera_bbox_info);
        fun(general_merge_info);
        fun(bbox_motion_info);
        fun(attribute_);
        fun(automobile_light_status);
        fun(right_edge_line);
        fun(non_automobile_vehicle_light_status);
        fun(lower_body_direction);
        fun(upper_body_direction);
        fun(face_direction);
        fun(intention);
        fun(depth_point);
        fun(od_rd_info);
        fun(debug_info);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(sensor_id);
        fun(label);
        fun(type_confidence);
        fun(existence_confidence);
        fun(general_object_probilibty);
        fun(sub_type);
        fun(track_id);
        fun(track_age_time_second);
        fun(last_measure_update_timestamp_ns);
        fun(current_prediction_timestamp_ns);
        fun(motion_status);
        fun(motion_unknown_score);
        fun(motion_static_score);
        fun(motion_stopped_score);
        fun(motion_moving_score);
        fun(motion_crossing_score);
        fun(maintenance_status);
        fun(occlusion_status);
        fun(sensor_source);
        fun(derived_type);
        fun(is_right_edge_detected);
        fun(is_with_ped);
        fun(umbrella);
        fun(call);
        fun(phone);
        fun(wave);
        fun(standing);
        fun(is_on_bike);
        fun(point_cloud_timestamp_ns);
        fun(track_range);
        fun(track_range_rate);
        fun(track_range_accel_rate);
        fun(track_lat_rate);
        fun(color_label);
        fun(sub_type_probs);
        fun(feature);
        fun(matched_sensor_object_info);
        fun(size_info);
        fun(polygon_box);
        fun(direction_info);
        fun(bv_corners);
        fun(motion_info);
        fun(camera_bbox_info);
        fun(general_merge_info);
        fun(bbox_motion_info);
        fun(attribute_);
        fun(automobile_light_status);
        fun(right_edge_line);
        fun(non_automobile_vehicle_light_status);
        fun(lower_body_direction);
        fun(upper_body_direction);
        fun(face_direction);
        fun(intention);
        fun(depth_point);
        fun(od_rd_info);
        fun(debug_info);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("sensor_id", sensor_id);
        fun("label", label);
        fun("type_confidence", type_confidence);
        fun("existence_confidence", existence_confidence);
        fun("general_object_probilibty", general_object_probilibty);
        fun("sub_type", sub_type);
        fun("track_id", track_id);
        fun("track_age_time_second", track_age_time_second);
        fun("last_measure_update_timestamp_ns", last_measure_update_timestamp_ns);
        fun("current_prediction_timestamp_ns", current_prediction_timestamp_ns);
        fun("motion_status", motion_status);
        fun("motion_unknown_score", motion_unknown_score);
        fun("motion_static_score", motion_static_score);
        fun("motion_stopped_score", motion_stopped_score);
        fun("motion_moving_score", motion_moving_score);
        fun("motion_crossing_score", motion_crossing_score);
        fun("maintenance_status", maintenance_status);
        fun("occlusion_status", occlusion_status);
        fun("sensor_source", sensor_source);
        fun("derived_type", derived_type);
        fun("is_right_edge_detected", is_right_edge_detected);
        fun("is_with_ped", is_with_ped);
        fun("umbrella", umbrella);
        fun("call", call);
        fun("phone", phone);
        fun("wave", wave);
        fun("standing", standing);
        fun("is_on_bike", is_on_bike);
        fun("point_cloud_timestamp_ns", point_cloud_timestamp_ns);
        fun("track_range", track_range);
        fun("track_range_rate", track_range_rate);
        fun("track_range_accel_rate", track_range_accel_rate);
        fun("track_lat_rate", track_lat_rate);
        fun("color_label", color_label);
        fun("sub_type_probs", sub_type_probs);
        fun("feature", feature);
        fun("matched_sensor_object_info", matched_sensor_object_info);
        fun("size_info", size_info);
        fun("polygon_box", polygon_box);
        fun("direction_info", direction_info);
        fun("bv_corners", bv_corners);
        fun("motion_info", motion_info);
        fun("camera_bbox_info", camera_bbox_info);
        fun("general_merge_info", general_merge_info);
        fun("bbox_motion_info", bbox_motion_info);
        fun("attribute_", attribute_);
        fun("automobile_light_status", automobile_light_status);
        fun("right_edge_line", right_edge_line);
        fun("non_automobile_vehicle_light_status", non_automobile_vehicle_light_status);
        fun("lower_body_direction", lower_body_direction);
        fun("upper_body_direction", upper_body_direction);
        fun("face_direction", face_direction);
        fun("intention", intention);
        fun("depth_point", depth_point);
        fun("od_rd_info", od_rd_info);
        fun("debug_info", debug_info);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("sensor_id", sensor_id);
        fun("label", label);
        fun("type_confidence", type_confidence);
        fun("existence_confidence", existence_confidence);
        fun("general_object_probilibty", general_object_probilibty);
        fun("sub_type", sub_type);
        fun("track_id", track_id);
        fun("track_age_time_second", track_age_time_second);
        fun("last_measure_update_timestamp_ns", last_measure_update_timestamp_ns);
        fun("current_prediction_timestamp_ns", current_prediction_timestamp_ns);
        fun("motion_status", motion_status);
        fun("motion_unknown_score", motion_unknown_score);
        fun("motion_static_score", motion_static_score);
        fun("motion_stopped_score", motion_stopped_score);
        fun("motion_moving_score", motion_moving_score);
        fun("motion_crossing_score", motion_crossing_score);
        fun("maintenance_status", maintenance_status);
        fun("occlusion_status", occlusion_status);
        fun("sensor_source", sensor_source);
        fun("derived_type", derived_type);
        fun("is_right_edge_detected", is_right_edge_detected);
        fun("is_with_ped", is_with_ped);
        fun("umbrella", umbrella);
        fun("call", call);
        fun("phone", phone);
        fun("wave", wave);
        fun("standing", standing);
        fun("is_on_bike", is_on_bike);
        fun("point_cloud_timestamp_ns", point_cloud_timestamp_ns);
        fun("track_range", track_range);
        fun("track_range_rate", track_range_rate);
        fun("track_range_accel_rate", track_range_accel_rate);
        fun("track_lat_rate", track_lat_rate);
        fun("color_label", color_label);
        fun("sub_type_probs", sub_type_probs);
        fun("feature", feature);
        fun("matched_sensor_object_info", matched_sensor_object_info);
        fun("size_info", size_info);
        fun("polygon_box", polygon_box);
        fun("direction_info", direction_info);
        fun("bv_corners", bv_corners);
        fun("motion_info", motion_info);
        fun("camera_bbox_info", camera_bbox_info);
        fun("general_merge_info", general_merge_info);
        fun("bbox_motion_info", bbox_motion_info);
        fun("attribute_", attribute_);
        fun("automobile_light_status", automobile_light_status);
        fun("right_edge_line", right_edge_line);
        fun("non_automobile_vehicle_light_status", non_automobile_vehicle_light_status);
        fun("lower_body_direction", lower_body_direction);
        fun("upper_body_direction", upper_body_direction);
        fun("face_direction", face_direction);
        fun("intention", intention);
        fun("depth_point", depth_point);
        fun("od_rd_info", od_rd_info);
        fun("debug_info", debug_info);
    }

    friend bool operator==(const ::CameraObjects& lhs, const ::CameraObjects& rhs) noexcept
    {
        return (lhs.sensor_id == rhs.sensor_id) && (lhs.label == rhs.label) && (fabs(static_cast<double>(lhs.type_confidence - rhs.type_confidence)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.existence_confidence - rhs.existence_confidence)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.general_object_probilibty - rhs.general_object_probilibty)) < DBL_EPSILON) && (lhs.sub_type == rhs.sub_type) && (lhs.track_id == rhs.track_id) && (lhs.track_age_time_second == rhs.track_age_time_second) && (lhs.last_measure_update_timestamp_ns == rhs.last_measure_update_timestamp_ns) && (lhs.current_prediction_timestamp_ns == rhs.current_prediction_timestamp_ns) && (lhs.motion_status == rhs.motion_status) && (fabs(static_cast<double>(lhs.motion_unknown_score - rhs.motion_unknown_score)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.motion_static_score - rhs.motion_static_score)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.motion_stopped_score - rhs.motion_stopped_score)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.motion_moving_score - rhs.motion_moving_score)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.motion_crossing_score - rhs.motion_crossing_score)) < DBL_EPSILON) && (lhs.maintenance_status == rhs.maintenance_status) && (lhs.occlusion_status == rhs.occlusion_status) && (lhs.sensor_source == rhs.sensor_source) && (lhs.derived_type == rhs.derived_type) && (lhs.is_right_edge_detected == rhs.is_right_edge_detected) && (lhs.is_with_ped == rhs.is_with_ped) && (lhs.umbrella == rhs.umbrella) && (lhs.call == rhs.call) && (lhs.phone == rhs.phone) && (lhs.wave == rhs.wave) && (lhs.standing == rhs.standing) && (lhs.is_on_bike == rhs.is_on_bike) && (lhs.point_cloud_timestamp_ns == rhs.point_cloud_timestamp_ns) && (fabs(static_cast<double>(lhs.track_range - rhs.track_range)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.track_range_rate - rhs.track_range_rate)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.track_range_accel_rate - rhs.track_range_accel_rate)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.track_lat_rate - rhs.track_lat_rate)) < DBL_EPSILON) && (lhs.color_label == rhs.color_label) && (lhs.sub_type_probs == rhs.sub_type_probs) && (lhs.feature == rhs.feature) && (lhs.matched_sensor_object_info == rhs.matched_sensor_object_info) && (lhs.size_info == rhs.size_info) && (lhs.polygon_box == rhs.polygon_box) && (lhs.direction_info == rhs.direction_info) && (lhs.bv_corners == rhs.bv_corners) && (lhs.motion_info == rhs.motion_info) && (lhs.camera_bbox_info == rhs.camera_bbox_info) && (lhs.general_merge_info == rhs.general_merge_info) && (lhs.bbox_motion_info == rhs.bbox_motion_info) && (lhs.attribute_ == rhs.attribute_) && (lhs.automobile_light_status == rhs.automobile_light_status) && (lhs.right_edge_line == rhs.right_edge_line) && (lhs.non_automobile_vehicle_light_status == rhs.non_automobile_vehicle_light_status) && (lhs.lower_body_direction == rhs.lower_body_direction) && (lhs.upper_body_direction == rhs.upper_body_direction) && (lhs.face_direction == rhs.face_direction) && (lhs.intention == rhs.intention) && (lhs.depth_point == rhs.depth_point) && (lhs.od_rd_info == rhs.od_rd_info) && (lhs.debug_info == rhs.debug_info);
    }
};


#endif // IMPL_TYPE_CAMERAOBJECTS_H
